@@ -26,6 +26,8 @@ from tropofy.file_io import FileIoConfig
 
 
 def main(global_config, **settings):
+    print("**************************************************************************************************")
+    print("Initialising server...")
     global_sqla_engine = settings.get('sqlalchemy.url', '')
 
     # If 'custom.use_app_schemas' not specified, use app schemas if a sqlalchemy.url is specified. 
@@ -108,12 +110,18 @@ def main(global_config, **settings):
     transaction.commit()
 
     if App.overridden_global_sqla_engine:
-        print("------------->You are connecting to a specific database: %s" % App.overridden_global_sqla_engine)
-        
+        print("You are connecting to a specific database: %s\n" % App.overridden_global_sqla_engine)
+
+    app_urls = [app.url_name for app in LoadedApps.apps.values()]
+    if len(app_urls) == 1:
+        print("**************************************************************************************************")
+        print("If you have not already, open up http://localhost:8080/ in your browser to login and view your app")
+    else:
+        print("Serving multiple apps on paths:")
+        for url in app_urls:
+            print("/%s" % url)
+
     print("**************************************************************************************************")
-    print("If you have not already, open up http://localhost:8080/ in your browser to login and view your app")
-    print("**************************************************************************************************")
-    
     return config.make_wsgi_app()
 
 
