@@ -16,6 +16,6 @@ class LoadedApps(object):
     apps = {}
 
 
-from data_set import AppDataSet, DataSetVar, SavedImageReference, FunctionExecutionInfo
+from data_set import AppDataSet, DataSetVar, SavedImageReference, CeleryTask, SynchronousTask, TaskResultMeta
 from application import App, AppWithDataSets, Parameter, Step, StepGroup
 from app_manager import AppManager
