@@ -41,6 +41,8 @@ class App(object):
         self.step_groups = []
         self.app_folder_path = None  # Needs to be set dynamically after app created. This is to avoid params in constructor.
         self._init_step_groups()
+        self.new_app_config_dict = {"has_none_template": True, "data_set_template_label": "Template"}
+        self.new_app_config_dict.update(self.get_new_dataset_config())
 
     def init_db_engine(self):
         """
@@ -75,6 +77,16 @@ class App(object):
     def get_examples(self):
         return {}
 
+    def get_new_dataset_config(self):
+        """
+        Return a dict of config options for the new dataset form
+        keys:
+            has_none_template- default True, type bool
+            data_set_template_label- default "Template", type string
+        :rtype: dict
+        """
+        return {}
+
     def get_example_app_names(self):
         return [k for k in self.get_examples()]
 
@@ -227,6 +239,13 @@ class AppWithDataSets(App):
         ))
         self.add_step_group(step_group)
 
+    def add_static_data_before_template_data_loaded(self, data_set):
+        """Populate data set with static data, before template data loaded.
+
+        :rtype: None
+        """
+        pass
+
     def create_new_data_set_for_user(self, user_email, example_data_set_name, specific_name=''):
         data_set = AppDataSet(self, user_email)
         if specific_name:
@@ -238,16 +257,21 @@ class AppWithDataSets(App):
 
         data_set.init_parameters_with_defaults()  # Must be done here and not in AppDataSet.__init__ as requires data set to have ID.
 
-        if example_data_set_name and self.get_examples():
+        # TODO: load_master_data flag ?
+        self.add_static_data_before_template_data_loaded(data_set)
+
+        return data_set
+
+    def populate_data_set_from_template(self, data_set, data_set_template_name):
+        if data_set_template_name and self.get_examples():
             try:
-                populate_example = self.get_examples().get(example_data_set_name)
+                populate_example = self.get_examples().get(data_set_template_name)
                 if populate_example:
                     populate_example(data_set)  # populates data set
             except Exception:
                 print(traceback.format_exc())
-                print("WARNING: An error occured loading your example data set. See Traceback above.")
+                print("WARNING: An error occurred loading your example data set. See Traceback above.")
 
-        return data_set
 
     def get_name(self):
         """This function returns the name of the App as a string.
