@@ -106,6 +106,8 @@ class AppDataSet(ORMBase):
         msg_strings = message if not isinstance(message, basestring) else [message]
         new_messages = []
         for msg in msg_strings:
+            if msg in ["<br>", "\r\n", "\n"]:
+                new_messages.append(TaskMessage(text="<br>", status=status))
             for msg_split_new_lines in re.split("<br>|\n", msg):  # A lot of code (often third party), adds newlines with <br> or \n explicitly into them. This will strip these and make new lines.
                 new_messages.append(TaskMessage(text=msg_split_new_lines, status=status))
 
@@ -147,12 +149,21 @@ class AppDataSet(ORMBase):
             if self.current_celery_task_terminated():  # TODO: Move into every 2 second loop?
                 subprocess.kill()
                 raise Exception()
-            lines.append(subprocess.stdout.readline())
-            if time.time() - last_update_time > 2:  # send_progress_message includes a db hit so is a bit slow. Sending less than every 2 seconds is useless, as this is polling frequency.
-                self.send_progress_message(lines, status=self.message_status.MUTED)
-                lines = []
-                last_update_time = time.time()
-        lines += subprocess.stdout.readlines()  # Add remaining lines
+
+            try:
+                lines.append(subprocess.stdout.readline()) # This is blocking see http://stackoverflow.com/questions/375427/non-blocking-read-on-a-subprocess-pipe-in-python
+                if time.time() - last_update_time > 2:  # send_progress_message includes a db hit so is a bit slow. Sending less than every 2 seconds is useless, as this is polling frequency.
+                    self.send_progress_message(lines, status=self.message_status.MUTED)
+                    lines = []
+                    last_update_time = time.time()
+            except AttributeError:
+                pass  # subprocess.stdout not set for subprocess.
+
+        try:
+            lines += subprocess.stdout.readlines()  # Add remaining lines
+        except AttributeError:
+            pass  # subprocess.stdout not set for subprocess.
+
         self.send_progress_message(lines, status=self.message_status.MUTED)
 
         errors = subprocess.stderr.readlines() if subprocess.stderr else None
@@ -363,7 +374,6 @@ class AppDataSet(ORMBase):
             self.set_var(name, value)
         else:
             raise ValidationException(name=name, value=value, message='Parameter does not exist.')
-        
 
     def init_parameters_with_defaults(self):  # Init parameter defaults on data set creation
         for param in self.app.get_parameters():
@@ -493,7 +503,7 @@ class TaskInterfaceMixin(object):
             'taskId': self.task_id,
             'dateTimeStartedStr': self.start_datetime,
             'elapsedSeconds': self.elapsed_seconds,
-            'progressMessagesHtml': "".join([m.to_html() for m in self.messages[-200:]]),
+            'progressMessagesHtml': "".join([m.to_html() for m in self.messages[-2000:]]),
             'isCurrentlyRunning': self.is_currently_running,
             'status': self.status,
             'taskCanBeTerminated': self.task_can_be_terminated(),
@@ -564,8 +574,10 @@ class CeleryTask(DataSetMixin, TaskInterfaceMixin):
 
     @property
     def end_datetime(self):
-        if self._result:
+        try:
             return self._result.end_datetime
+        except AttributeError:
+            pass  # self._result is None. (Warning! Check self._result and then returning self._result.end_datetime can cause an error as self._result is changed in another process)
 
     @property
     def status(self):
