@@ -51,8 +51,8 @@ class ClassTabularDataMapping(object):
 
     @staticmethod  # Use staticmethod as can't have cls as first attribute. Used as process_objects arg in ClassWsMapping.
     def process_sqla_objects(data_set, objects):
-        DBSession().add_all(objects)
-        DBSession.flush()
+        data_set.add_all(objects)
+        data_set.flush()
 
     @classmethod  # Use staticmethod as can't have cls as first attribute. Used as process_objects arg in ClassWsMapping.
     def get_all_sqla_objects_of_class_in_data_set(cls, data_set, class_):
