@@ -40,6 +40,23 @@ class ClassTabularCsvMapping(ClassTabularDataMapping):
 class CsvReader(object):
     """Utility class for reading data from .csv files."""
 
+    @classmethod
+    def load_tabular_data_from_csv_file_in_memory(cls, data_set, file_in_memory, class_tabular_csv_mapping, delimiter=',', user_feedback_csv_file_identifier_str=''):
+        """Loads data from a csv in memory file in a table format. Assumes first row is headings, and each subsequent row is data corresponding to the headings.
+
+        :param data_set: Data set used to store the data read in.
+        :type data_set: :class:`tropofy.app.AppDataSet`
+        :param file_in_memory: File in memory. Common use case is to pass from :func:`tropofy.widget.file_upload_widget.process_file`
+        :type file_in_memory: FileIO
+        :param class_tabular_csv_mapping: Describes how the data and headings in the csv file map to a class.
+        :type class_tabular_csv_mapping: :class:`tropofy.file_io.read_write_csv.ClassTabularCsvMapping`
+        :param delimiter: (Optional) csv delimiter. Default ','
+        :type delimiter: str
+        :param user_feedback_csv_file_identifier_str: String to identify the csv file in error reporting.
+        :ty[e user_feedback_csv_file_identifier_str: str
+        """
+        cls._load_tabular_data_from_csv_file(data_set, file_in_memory.file, class_tabular_csv_mapping, delimiter, user_feedback_csv_file_identifier_str)
+
     @classmethod
     def load_tabular_data_from_csv_file_on_disk(cls, data_set, file_path, class_tabular_csv_mapping, delimiter=','):
         """Loads data from a csv file in a table format. Assumes first row is headings, and each subsequent row is data corresponding to the headings.
@@ -48,24 +65,24 @@ class CsvReader(object):
         :type data_set: :class:`tropofy.app.AppDataSet`
         :param file_path: Path on disk to the csv file.
         :type file_path: str
-        :param class_csv_with_headings_mapping: Describes how the data and headings in the csv file map to a class.
-        :type class_csv_with_headings_mapping: :class:`tropofy.file_io.read_write_csv.ClassTabularCsvMapping`
+        :param class_tabular_csv_mapping: Describes how the data and headings in the csv file map to a class.
+        :type class_tabular_csv_mapping: :class:`tropofy.file_io.read_write_csv.ClassTabularCsvMapping`
         :param delimiter: (Optional) csv delimiter. Default ','
         :type delimiter: str
         """
-        csv_file = open(file_path, 'rb')
+        cls._load_tabular_data_from_csv_file(data_set, open(file_path, 'rb'), class_tabular_csv_mapping, delimiter, file_path)
+
+    @classmethod
+    def _load_tabular_data_from_csv_file(cls, data_set, csv_file, class_tabular_csv_mapping, delimiter=',', user_feedback_csv_file_identifier_str=''):
         csv_dict_reader = csv.DictReader(csv_file, delimiter=delimiter)
-        header_row = csv_dict_reader.fieldnames
-
-        if class_tabular_csv_mapping.tabular_data_has_required_columns(header_row, file_path):
-            def process_dict_rows(data_set, rows):
-                for row in rows:
-                    attribute_values = dict((class_tabular_csv_mapping.get_attribute_from_column_name(col_heading), value) for col_heading, value in row.iteritems())
-                    data_set.add(class_tabular_csv_mapping.class_(
-                        **attribute_values
-                    ))
-
-            process_dict_rows(data_set, csv_dict_reader)
+        if class_tabular_csv_mapping.tabular_data_has_required_columns(csv_dict_reader.fieldnames, user_feedback_csv_file_identifier_str):
+            objects = []
+            for row in csv_dict_reader:
+                attribute_values = dict((class_tabular_csv_mapping.get_attribute_from_column_name(col_heading), value) for col_heading, value in row.iteritems())
+                objects.append(class_tabular_csv_mapping.class_(
+                    **attribute_values
+                ))
+            class_tabular_csv_mapping.process_objects(data_set, objects)
 
 
 class CsvWriter(object):
@@ -82,8 +99,8 @@ class CsvWriter(object):
         :type data_set: :class:`tropofy.app.AppDataSet`
         :param file_path: Path on disk to the csv file.
         :type file_path: str
-        :param class_csv_with_headings_mapping: Describes how a class maps to headings and data in the csv file.
-        :type class_csv_with_headings_mapping: :class:`tropofy.file_io.read_write_csv.ClassTabularCsvMapping`
+        :param class_tabular_csv_mapping: Describes how a class maps to headings and data in the csv file.
+        :type class_tabular_csv_mapping: :class:`tropofy.file_io.read_write_csv.ClassTabularCsvMapping`
         :param delimiter: (Optional) csv delimiter. Default ','
         :type delimiter: str
         """
