@@ -13,16 +13,17 @@ or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 
 import inspect
 
-from widgets import Widget
+from widgets import Widget, WidgetConfigException
+from form_widget.form_widget import Form  # Needs to be before GridWidget
+from file_upload_widget.file_upload_widget import FileUpload  # Used by DataSets. Import order is important.
 from grid_widget.grid_widget import GridWidget
 from grid_widget.simple_grid import SimpleGrid
-from file_upload_widget.file_upload_widget import FileUpload  # Used by DataSets. Import order is important.
 from kml_map_widget.kml_map_widget import KMLMap
 from chart_widget.chart_widget import Chart, FilteredChart
 from execute_function_widget.execute_function_widget import ExecuteFunction
 from image_widget.image_widget import StaticImage
 from custom_export_widget.custom_export_widget import CustomExport, ExportDataSet
 from download_file_from_url_widget.download_file_from_url_widget import DownloadFileFromUrl
-from form_widget.form_widget import Form, ParameterForm, Filter, FilterMixin
+from form_widget.form_widget import ParameterForm, Filter, FilterMixin
 from download_file_from_url_widget.download_file_from_url_widget import DownloadFileFromUrl
 from data_sets_widget.data_sets_widget import DataSets
