@@ -26,7 +26,7 @@ class CustomExport(Widget):
 
     def refresh_from_db(self, data_set, request):
         if data_set:
-            return {'get_custom_export_data': self.get_custom_export_data(data_set)}
+            return {'success': True, 'data': self.get_custom_export_data(data_set)}
 
     def update_from_db(self, data, data_set_id, oper='', data_set=None):
         response = Response(self.get_custom_export_data(data_set))
