@@ -1,4 +1,4 @@
-'''
+"""
 Author:      www.tropofy.com
 
 Copyright 2013 Tropofy Pty Ltd, all rights reserved.
@@ -9,12 +9,12 @@ available at: http://www.tropofy.com/terms_of_service.html
 This source file is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
-'''
+"""
 
 from datetime import datetime, date, time
 from sqlalchemy.exc import IntegrityError
 from tropofy.database.tropofy_orm import ValidationException, Validator
-from tropofy.widgets.widgets import Widget
+from tropofy.widgets.widgets import Widget, WidgetConfigException
 
 
 class Form(Widget):
@@ -28,9 +28,9 @@ class Form(Widget):
 
         :param name: Form element name. Used to reference the value of the element on submit.
         :type name: str
-        :param input_type: Type of form input. ('text', 'select', 'date', 'time') 
+        :param input_type: Type of form input. (Form.Element.TEXT, Form.Element.SELECT, Form.Element.DATE, Form.Element.TIME, Form.Element.DATETIME, Form.Element.BOOL)
         :type input_type: str
-        :param default: Default input value. If 'options' provided, default must be an option.
+        :param default: (optional) Default input value. Default defaults are: TEXT:"",  SELECT: value of first option. If SELECT element, default must be an option value.
         :type default: Multiple possible types. Must be valid relative to options list and input_type.
         :param label: (optional) Label of form element.
         :type label: str
@@ -38,8 +38,8 @@ class Form(Widget):
         :type hidden: bool
         :param disabled: (optional) If true, form element will be disabled
         :type disabled: bool
-        :param options: (optional - used if input_type=='select') Valid options that this element may take.
-        :type options: list of strings or list of :class:`tropofy.widgets.Form.Element.Option`
+        :param options: (optional - used if input_type=='select', or 'typeahead') for select valid options that this element may take, for typeahead suggested options that this element may take
+        :type options: for select expects list of strings or list of :class:`tropofy.widgets.Form.Element.Option`, for typeahead expects a list of strings
         :param compute_options_from: (optional - used if input_type=='select' and options=None). String matching the `name` of a dependent_option_list of a :class:`tropofy.Form.Element.Option` within a :class:`tropofy.Form.Element` in this widget.
         :type compute_options_from: str
         """
@@ -47,34 +47,39 @@ class Form(Widget):
         SELECT = 'select'
         DATE = 'date'
         TIME = 'time'
+        DATETIME = 'datetime'
+        BOOL = 'bool'
+        TYPEAHEAD = 'typeahead'
 
-        valid_input_types = ['text', 'select', 'date', 'time']
+        VALID_INPUT_TYPES = [TEXT, SELECT, DATE, TIME, DATETIME, BOOL, TYPEAHEAD]
+        VALID_DATETIME_INPUT_TYPES = {DATE, TIME, DATETIME}
+
+        def __init__(self, name, input_type, default=None, label=None, hidden=False, disabled=False, options=None, compute_options_from=None):
+            if input_type == Form.Element.BOOL:
+                input_type = Form.Element.SELECT
+                options = [
+                    Form.Element.Option(value=True, text='True'),
+                    Form.Element.Option(value=False, text='False'),
+                ]
+                default = default if default is not None else True
+                if not isinstance(default, bool):
+                    raise Form.FormElementException("Invalid default for boolean: '%s'. Must be either True or False" % default)
 
-        def __init__(self, name, input_type, default, label=None, hidden=False, disabled=False, options=None, compute_options_from=None):
             self.name = name
-
-            # Validation
-            if input_type not in Form.Element.valid_input_types:
-                Form.FormElementException("Invalid input type '%s'. Must be one of %s" % (input_type, str(Form.Element.valid_input_types)))            
-
-            if options and default not in options:
-                Form.FormElementException("Default value '%s', must be one of %s" % (default, str(options)))
-
-            self.input_type = input_type
+            self.input_type = input_type  # Must be defined by options. Options validator uses this.
+            self.options = options  # Must be defined before default_value. Default validator uses this.
             self.default_value = default
             self.hidden = hidden
             self.disabled = disabled
             self.label = label if label is not None else self.name
-
-            self.options = self._process_options(options) if options else []  # With _process_options, allow old types for backward compatibility (would change app code)
-            self.compute_options_from = compute_options_from if self.options == [] and self.is_select else None
+            self.compute_options_from = compute_options_from if self.options == [] and self.is_select_element else None
 
         def serialise(self):
             return {
                 'name': self.name,
                 'inputType': self.input_type,
                 'defaultValue': self.default_value,
-                'options': [o.__json__() for o in self.options],
+                'options': [o.serialise() for o in self.options] if self.is_select_element else self.options,
                 'computeOptionsFrom': self.compute_options_from,
                 'label': self.label,
                 'hidden': self.hidden,
@@ -82,8 +87,73 @@ class Form(Widget):
             }
 
         @property
-        def is_select(self):
-            return self.input_type == 'select'
+        def is_select_element(self):
+            return self.input_type == Form.Element.SELECT
+
+        @property
+        def is_typeahead_element(self):
+            return self.input_type == Form.Element.TYPEAHEAD
+
+        ####### INPUT VALIDATORS #############################
+        @property
+        def input_type(self):
+            return self._input_type
+
+        @input_type.setter
+        def input_type(self, value):
+            if value not in Form.Element.VALID_INPUT_TYPES:
+                raise Form.FormElementException("Invalid input type '%s'. Must be one of %s" % (value, str(Form.Element.VALID_INPUT_TYPES)))
+            self._input_type = value
+
+        @property
+        def options(self):
+            return self._options
+
+        @options.setter
+        def options(self, value):
+            self._options = []
+            if self.is_select_element:
+                for option in value:
+                    if isinstance(option, Form.Element.Option):
+                        self._options.append(option)
+                    else:
+                        if isinstance(option, basestring):
+                            option = {'value': option}
+                        try:
+                            self._options.append(Form.Element.Option(**option))
+                        except:
+                            raise Form.FormElementException('Could not create Form.Element.Option from %s. You should create Form.Element.Option objects directly in your app. Refer to docs for details.' % option)
+                if len(self._options) < 1:
+                    raise Form.FormElementException("Select element '%s' has no options. Please pass a list of options." % self.name)
+
+            elif self.is_typeahead_element:
+                self._options = value
+
+        @property
+        def default_value(self):
+            return self._default_value
+
+        @default_value.setter
+        def default_value(self, value):
+            if value is None:
+                if self.input_type in [Form.Element.TEXT, Form.Element.TYPEAHEAD]:
+                    value = ""
+
+                elif self.input_type in [Form.Element.DATE, Form.Element.TIME, Form.Element.DATETIME]:
+                    value = ""
+                elif self.is_select_element:
+                    value = self.options[0].value  # Checked earlier that if SELECT element, at least one option.
+                else:
+                    raise Form.FormElementException("Default value not provided, and no 'default default' for form element type %s" % self.input_type)
+
+            if self.is_select_element:
+                option_values = [o.value for o in self.options]
+                if value not in option_values:
+                    raise Form.FormElementException("Default for `select` element exception. Default value '%s' not found in options values: %s" % (value, str(option_values)))
+
+            self._default_value = value
+        ######## END INPUT VALIDATORS #########################
+
 
         def _process_options(self, options):
             """Convert to list of :class:`tropofy.widgets.Form.Element.Option` from option formats:
@@ -93,19 +163,8 @@ class Form(Widget):
 
             :rtype: list of :class:`tropofy.widgets.Form.Element.Option`
             """
-            converted_options = []
-            for option in options:
-                if isinstance(option, Form.Element.Option):
-                    converted_options.append(option)
-                else:
-                    if isinstance(option, basestring):
-                        option = {'value': option}
-                    try:
-                        converted_options.append(Form.Element.Option(**option))
-                    except:
-                        raise Exception('Could not create Form.Element.Option from %s. You should create Form.Element.Option objects directly in your app. Refer to docs for details.' % option)
 
-            return converted_options
+
 
         class Option(object):
             """An option of a form select.
@@ -122,13 +181,13 @@ class Form(Widget):
                 self.text = text if text else self.value
                 self.dependent_option_lists = dependent_option_lists if dependent_option_lists else {}
 
-            def __json__(self):
+            def serialise(self):
                 return {
                     'value': self.value,
                     'text': self.text,
                     'dependentOptionLists': [{
                         'name': name,
-                        'options': [o.__json__() for o in options]
+                        'options': [o.serialise() for o in options]
                     } for name, options in self.dependent_option_lists.iteritems()]
                 }
 
@@ -142,9 +201,10 @@ class Form(Widget):
         return {
             'formElements': [e.serialise() for e in form_elements],
             'formTitle': self.get_title(data_set),
-            'autoSubmit': self.is_autosubmit(data_set),
-            'autoSubmitEntireForm': self.autosubmit_entire_form(data_set),
+            'isAutoSubmit': self.is_autosubmit(data_set),
+            'isSubmitEntireForm': self.is_submit_entire_form(data_set),
             'hideSubmitButton': self.hide_submit_button(data_set),
+            'datetimeDisplayFormats': self._get_datetime_display_formats(),
         }
 
     def get_form_elements(self, data_set):
@@ -172,7 +232,7 @@ class Form(Widget):
 
         'name', 'success', 'message', 'value'
         :rtype: dict
-        """  # Todo: Probably add another element in returns for 'edited_row' so can integrate with Grid.
+        """
         raise NotImplementedError
 
     def get_title(self, data_set):
@@ -198,8 +258,8 @@ class Form(Widget):
         """
         return True
 
-    def autosubmit_entire_form(self, data_set):  # Todo: Merge with is_autosubmit to return dict of options.
-        """Determines whether a form on autosubmits submits the entire form. Does nothing if :func:`tropofy.widgets.Form.is_autosubmit` returns False.
+    def is_submit_entire_form(self, data_set):
+        """Determines whether a form on submit submits the entire form.
 
         Default False - means only form element that has changed is submitted.
         :param data_set: current data set.
@@ -223,17 +283,50 @@ class Form(Widget):
 
     def update_to_db(self, data, data_set_id, oper, data_set):
         form_elements = self.get_form_elements(data_set)
+        results = []
         for element in [e for e in form_elements if data.get(e.name)]:
-            if element.input_type == Form.Element.DATE:
-                data[element.name] = Validator.validate(data[element.name], date, element.name)
-            elif element.input_type == Form.Element.TIME:
-                data[element.name] = Validator.validate(data[element.name], time, element.name)
-
-        results = self.process_data(data, data_set)
+            try:
+                if element.input_type == Form.Element.DATE:
+                    data[element.name] = Validator.validate(data[element.name], date, element.name)
+                elif element.input_type == Form.Element.TIME:
+                    data[element.name] = Validator.validate(data[element.name], time, element.name)
+                elif element.input_type == Form.Element.DATETIME:
+                    data[element.name] = Validator.validate(data[element.name], datetime, element.name)
+            except (ValidationException, IntegrityError) as e:
+                results.append({
+                    'name': element.name,
+                    'success': False,
+                    'message': e.message if e.message else 'A validation error occurred.'
+                })
+        if not results:  # No exception
+            results = self.process_data(data, data_set)
         return results if results else ""
 
+    def _get_datetime_display_formats(self):
+        formats = {
+            Form.Element.DATE: "DD/MM/YYYY",
+            Form.Element.DATETIME: "HH:mm DD/MM/YYYY",
+            Form.Element.TIME: "HH:mm"
+        }
+        custom_formats = self.get_datetime_display_formats()
+        if set(custom_formats.keys()) > Form.Element.VALID_DATETIME_INPUT_TYPES:
+            raise WidgetConfigException("Error in function Form.get_datetime_display_formats. Must return dict with keys in: [Form.Element.DATE, Form.Element.TIME, Form.Element.DATETIME]")
+        formats.update(custom_formats)
+        return formats
+
+    def get_datetime_display_formats(self):
+        """ Specify custom display formats to display datetimes, dates, and times in a form.
+
+        Use format strings specified by `MomentJS <http://momentjs.com/docs/#/parsing/string-format/>`_. Note that these are slightly different to string-formats used in Python.
+
+        Allowable keys are: Form.Element.DATE, Form.Element.DATETIME, and Form.Element.DATE
+
+        :returns: dict
+        """
+        return {}
+
     @classmethod
-    def get_input_type_from_python_type(cls, input_type):
+    def _get_input_type_from_python_type(cls, input_type):
         """Helper function to match python types with form input type.
 
         Useful in :func:`tropofy.widgets.Form.get_form_elements`, when creating :class:`tropofy.widgets.Form.Element`
@@ -244,15 +337,15 @@ class Form(Widget):
         :rtype: str
         """
         type_to_form_type = {
-            int: 'text',
-            str: 'text',
-            float: 'text',
-            bool: 'select',
-            date: 'date',
-            time: 'time',
-            datetime: 'datetime',
+            int: cls.Element.TEXT,
+            str: cls.Element.TEXT,
+            float: cls.Element.TEXT,
+            bool: cls.Element.BOOL,
+            date: cls.Element.DATE,
+            time: cls.Element.TIME,
+            datetime: cls.Element.DATETIME,
         }
-        return type_to_form_type.get(input_type)
+        return type_to_form_type.get(input_type, Form.Element.TEXT)
 
 
 class ParameterForm(Form):
@@ -270,18 +363,11 @@ class ParameterForm(Form):
         for param in data_set.app.get_parameters():
             if not self.parameter_names_filter or param.name in self.parameter_names_filter:
                 value = data_set.get_param(param.name)
-
                 options = param.options
-                if param.allowed_type is bool:
-                    options = [
-                        Form.Element.Option(value=True, text='True'),
-                        Form.Element.Option(value=False, text='False'),
-                    ]
-                    value = 1 if value else 0
 
                 elements.append(Form.Element(
                     name=param.name,
-                    input_type='select' if options else ParameterForm.get_input_type_from_python_type(type(value)),
+                    input_type=Form.Element.SELECT if options else ParameterForm._get_input_type_from_python_type(param.allowed_type if param.allowed_type else type(value)),
                     default=value,
                     label=param.label,
                     options=options,
@@ -353,7 +439,7 @@ class Filter(Form):
     def _get_var_name(self, element_name):
         return 'filter_%s_%s' % (self.id, element_name)
 
-    def autosubmit_entire_form(self, data_set):
+    def is_submit_entire_form(self, data_set):
         return True
 
     def refresh_from_db(self, data_set, request):
