@@ -132,7 +132,7 @@ class MapNodeInput(KMLMap):
     def get_kml(self, data_set):
         kml = Kml()
 
-        node_style = Style(iconstyle=IconStyle(scale=0.8, icon=Icon(href='http://maps.google.com/mapfiles/kml/paddle/blu-circle-lv.png')))
+        node_style = Style(iconstyle=IconStyle(scale=0.8, icon=Icon(href='https://maps.google.com/mapfiles/kml/paddle/blu-circle-lv.png')))
         node_folder = kml.newfolder(name="Nodes")
         for p in [node_folder.newpoint(name=n.name, coords=[(n.longitude, n.latitude)]) for n in data_set.query(Node).all()]:
             p.style = node_style
@@ -150,7 +150,7 @@ class MapArcInput(KMLMap):
     def get_kml(self, data_set):
         kml = Kml()
 
-        node_style = Style(iconstyle=IconStyle(scale=0.8, icon=Icon(href='http://maps.google.com/mapfiles/kml/paddle/blu-circle-lv.png')))
+        node_style = Style(iconstyle=IconStyle(scale=0.8, icon=Icon(href='https://maps.google.com/mapfiles/kml/paddle/blu-circle-lv.png')))
         node_folder = kml.newfolder(name="Nodes")
         for p in [node_folder.newpoint(name=n.name, coords=[(n.longitude, n.latitude)]) for n in data_set.query(Node).all()]:
             p.style = node_style
@@ -176,7 +176,7 @@ class MapFlowOutput(KMLMap):
 
         kml = Kml()
 
-        node_style = Style(iconstyle=IconStyle(scale=0.8, icon=Icon(href='http://maps.google.com/mapfiles/kml/paddle/blu-circle-lv.png')))
+        node_style = Style(iconstyle=IconStyle(scale=0.8, icon=Icon(href='https://maps.google.com/mapfiles/kml/paddle/blu-circle-lv.png')))
         node_folder = kml.newfolder(name="Nodes")
         for p in [node_folder.newpoint(name=n.name, coords=[(n.longitude, n.latitude)]) for n in data_set.query(Node).all()]:
             p.style = node_style
